#include "DataRange.h"

char DataRange::_nInstances = 0;

DataRange::DataRange(const Role &r) {
    if (_nInstances + 1 > 2) {
        throw std::out_of_range("Only two instances allowed");
    }
    ++_nInstances;
    _collectionRole = r;
}

int_fast64_t DataRange::operator*(const DataRange &rhs) const {
    int_fast64_t result = _range;
    if (*this == rhs) {
        result *= (_range - 1);
        result /= 2;
    } else {
        result *= rhs._range;
    }
    return result;
}

bool DataRange::overlapsButNotEqualTo(const DataRange &rhs) const {

    if (_start < rhs._start)
        return _start + _range > rhs._start;
    else if (_start > rhs._start) {
        return rhs._start + rhs._range > _start;
    }

    return rhs._range != _range;
}

void DataRange::crossValidateWith(DataRange &rhs) {
    if (_collectionRole == rhs._collectionRole)
        throw std::logic_error("Both objects have the same role");
    if (_start > rhs._start && _collectionRole == Role::Rows) {
        throw std::domain_error("Rows and columns swapped");
    }
    if (_start < rhs._start && _collectionRole == Role::Columns) {
        throw std::domain_error("Rows and columns swapped");
    }
    if (overlapsButNotEqualTo(rhs))
        throw std::range_error("Ranges for rows and columns OVERLAP");

    _crossValidated = true;
    rhs._crossValidated = true;

}

int_fast64_t DataRange::operator^(const DataRange &rhs) const {
    int_fast64_t tmp = _range;
    return tmp * rhs._range;
}
