#ifndef SLIDER_DATARANGE_H
#define SLIDER_DATARANGE_H

#include <stdexcept>
#include <cstdint>

// DataRange represents range of residues from the input file
class DataRange {
public:
    enum class Role {
        Rows, Columns, Undefined
    };

private:
    static char _nInstances;
    unsigned _start = 0;
    unsigned _range = 0;
    bool _crossValidated = false;
    Role _collectionRole = Role::Undefined;

public:
    DataRange(const Role &);

    inline void setStart(const unsigned int &start) {
        _start = start;
    }

    inline unsigned getStart() {
        return _start;
    }

    inline void setRange(const unsigned int &range) {
        _range = range;
    }

    inline unsigned getRange() {
        return _range;
    }

    inline bool operator==(const DataRange &rhs) const {
        return _start == rhs._start &&
               _range == rhs._range;
    }

    // check if overlap exists
    bool overlapsButNotEqualTo(const DataRange &rhs) const;

    inline bool operator%(const DataRange &rhs) const {
        return _start == 0 && rhs._start == 0;
    }

    void crossValidateWith(DataRange &rhs);

    inline bool isCrossValidated() {
        return _crossValidated;
    }

    // This operator returns the number of comparisons between collections
    int_fast64_t operator*(const DataRange &rhs) const;

    // This operator returns result buffer size
    int_fast64_t operator^(const DataRange &rhs) const;
};

#endif //SLIDER_DATARANGE_H
