#ifndef SLIDER_DOTPLOTCOLLECTION_H
#define SLIDER_DOTPLOTCOLLECTION_H

#include <cstring>
#include "utils.h"
#include "DotPlot.h"
#include "DataRange.h"
#include "DotPlotInfoRepository.h"

class DotPlotCollection {
private:
    DotPlot *_data = nullptr;
    off64_t *_contents = nullptr;
    DataRange *_dataRange = nullptr;
    bool _reference = false;

    // static section
    static const char *_header;
    static unsigned _nDotPlots;
    static unsigned long _nResults;
    static off64_t _contentsPosition;
    static ushort _maxSeqLen[2];
    static ushort _maxNUniq;
    static std::string _dataFileName;
    static char *_outFilePrefix;
    static FILE *_fpDataFile;
    static int _fdescDataFile;
    static int _nThreads;

public:
    const static uint DP_VERSION;
    const static unsigned char RECORD_START;
    enum sequenceLength {
        first, second
    };

    DotPlotCollection() = default;

    DotPlotCollection(const DotPlotCollection &dr) = delete;

    ~DotPlotCollection();

    DotPlotCollection &operator=(const DotPlotCollection &rhs);

    inline void setDataRange(DataRange *dataRange) {
        if (dataRange->isCrossValidated()) {
            if (dataRange->getStart() + dataRange->getRange() > _nDotPlots) {
                fprintf(stderr, "!!! More dot plots in range (%d) than present in the file (%d)\n",
                        dataRange->getRange(), _nDotPlots);
                throw std::runtime_error("Inconsistent input");
            }
            _dataRange = dataRange;
        } else
            throw std::domain_error("Ranges must be validated before assignment to collections");
    };

    void writeResults(const float *r, const unsigned &rowStart, const unsigned &colStart, const unsigned &nRows,
                      const unsigned &nCols);

    void buildFromInfoRepository(DotPlotInfoRepository &dpir);

    static void readCollectionHeader();

    static void writeCollectionHeader();


    static void prepareComparisonMatrix();

    static void checkComparisonMatrix();


    static int_fast64_t nTotalJobs();


    static std::string getComparisonMatrixFileName();

    inline static void setComparisonMatrixFileNamePrefix(char *fnOut) {
        _outFilePrefix = fnOut;
    }

    inline static void setNThreads(const int &nt) {
        Utils::check_threads(nt);
        _nThreads = nt;
    };

    inline static const int &getNThreads() {
        return _nThreads;
    }

    inline static void setDataFileName(char *fnDat) {
        _dataFileName = fnDat;
    }

    inline static void setDataFileName(std::string s) {
        _dataFileName = s;
    }

    void populate();

    void compare(const DotPlotCollection &rhs);
};

#endif //SLIDER_DOTPLOTCOLLECTION_H
