#ifndef SLIDER_DOTPLOTINFO_H
#define SLIDER_DOTPLOTINFO_H

#include <string>
#include <vector>


struct DotPlotInfoItem {
    std::string id;
    std::string path;
    size_t fileSize = 0;
    unsigned short seqLen = 0;
    unsigned short nUniq = 0;
    unsigned short windowSize = 0;
    unsigned int nPoints = 0;
    float nPointsPerResidue = 0;
    float blackScore = 0;
    float logScore = 0;
};

enum DotPlotInfoItemIndex {
    DP_ID,
    DP_PATH,
    DP_FS,
    DP_SEQL,
    DP_NPOINTS,
    DP_NUNIQ,
    DP_WINSIZE,
    DP_NPOINTSPERRES,
    DP_BLACKSCORE,
    DP_LOGSCORE,
    DP_NR
};

class DotPlotInfoRepository {
private:
    const char *repoFileName;
    std::vector<DotPlotInfoItem> dotPlotInfoCollection;
    DotPlotInfoItemIndex lastColumn;
    size_t nDotPlots;
    size_t rawDataSize;
    int_fast64_t totalABCSize;

public:
    DotPlotInfoRepository(const char *, const DotPlotInfoItemIndex &);

    void process();

    DotPlotInfoItem &operator[](size_t index);

    const size_t &getNDotPlots() const;

    const size_t &getRawDataSize() const;
};


#endif //SLIDER_DOTPLOTINFO_H
