#include <iostream>
#include <omp.h>
#include <getopt.h>
#include "utils.h"
#include "DotPlotInfoRepository.h"
#include "DotPlotCollection.h"


class dotc_options {
public:
    char *inputFileName = nullptr;
    char *outputPrefix = nullptr;
    int nThreads = 1;
};

static void usage() {
    printf(
            "\n"
            "DOTC – Compile dot plot collection for use with SLIDER\n"
            "-i,    --input-list,    Protein sequence database generated by dotrun script\n"
            "-o,    --output,        Binary dot plot collection for SLIDER\n"
            "-t,    --nthreads,      Number of threads to use\n"
            "-h,    --help,          Print this help\n"
    );
}

int main(int argc, char **argv) {

    dotc_options opts;

    static struct option long_options[] = {
            {"input-list", required_argument, 0, 'i'},
            {"output",     required_argument, 0, 'o'},
            {"nthreads",   optional_argument, 0, 't'},
            {"help",       optional_argument, 0, 'h'},
            {0, 0,                            0, 0}
    };

    printf("*** DOTC ***\n");
    Utils::print_license();
    Utils::print_citation();
    Utils::print_copyright();

    int long_index = 0, opt;
    while ((opt = getopt_long(argc, argv, "i:o:t:h",
                              long_options, &long_index)) != -1) {
        switch (opt) {
            case 'i':
                opts.inputFileName = optarg;
                break;
            case 'o':
                opts.outputPrefix = optarg;
                break;
            case 't':
                opts.nThreads = atoi(optarg);
                break;
            case 'h':
                usage();
                exit(EXIT_SUCCESS);
            default:
                printf("No such option %c", opt);
                usage();
                exit(EXIT_FAILURE);
        }
    }

    Utils::check_threads(opts.nThreads);

    DotPlotInfoRepository dpInfo = DotPlotInfoRepository(opts.inputFileName, DotPlotInfoItemIndex::DP_FS);
    dpInfo.process();

    DotPlotCollection dpc;
    DotPlotCollection::setNThreads(opts.nThreads);
    DotPlotCollection::setDataFileName(Utils::generateFileName(opts.outputPrefix, nullptr, "bin"));

    dpc.buildFromInfoRepository(dpInfo);

    return 0;
}
