#include "utils.h"

std::string Utils::generateFileName(const char *prefix, const char *suffix, const char *ext) {
    std::stringstream s;
    s << prefix;
    if (suffix != nullptr)
        s << "-" << suffix;
    s << "." << ext;
    return s.str();
}

off64_t Utils::getFilesize(const char *filename) {
    struct stat stat_buf;
    int rc = stat(filename, &stat_buf);
    if (rc != 0)
        perror("Problem reading file size");
    return stat_buf.st_size;
}

void Utils::jobIndexToDotPlotIndices(const size_t &nDotPlots, const int_fast64_t &jobIndex, uint &row, uint &col) {
    static const size_t y = 2 * nDotPlots - 1;
    static const unsigned long long y2 = y * y;
    row = static_cast<uint>(std::floor(0.5 * (double(y) - std::sqrt(y2 - 8 * jobIndex))));
    const uint &n = row;
    col = static_cast<uint>(jobIndex + n + 1 - (y - n) * n / 2);
}

void Utils::printInfo(int argc, char *const *argv, int nWorld) {
    fprintf(stdout, "Command line was:\n");
    for (int m = 0; m < argc; m++)
        fprintf(stdout, "%s ", argv[m]);
    fprintf(stdout, "\n");
    fprintf(stdout, "World size is %d\n", nWorld);
    fflush(stdout);
}

void Utils::check_threads(const int &expectedNThreads) {
    omp_set_num_threads(expectedNThreads);
#pragma omp parallel
    {
        if (expectedNThreads > omp_get_max_threads()) {
            fprintf(stderr, "!!! You can request at most %d threads", omp_get_num_threads());
            throw std::runtime_error("Too many threads requested!");
        }
    }


}

void Utils::get_proc_bind_info() {
    fprintf(stdout, "$$$ Setting manually OMP_PLACES and OMP_PROC_BIND may improve performance.\n");
    omp_proc_bind_t proc_bind = omp_get_proc_bind();
    char proc_bind_str[16];
    switch (proc_bind) {
        case omp_proc_bind_false:
            strcpy(proc_bind_str, "FALSE");
            break;
        case omp_proc_bind_true:
            strcpy(proc_bind_str, "TRUE");
            break;
        case omp_proc_bind_master:
            strcpy(proc_bind_str, "MASTER");
            break;
        case omp_proc_bind_close:
            strcpy(proc_bind_str, "CLOSE");
            break;
        case omp_proc_bind_spread:
            strcpy(proc_bind_str, "SPREAD");
            break;
        default:
            // This should be never hit
            fprintf(stderr, "Unknown value of omp_proc_bind_t: %d\n", proc_bind);
            exit(EXIT_FAILURE);
    }
    fprintf(stdout, "OMP_PROC_BIND = %s\n", proc_bind_str);
    if (proc_bind == omp_proc_bind_false) {
        fprintf(stdout, "Warning: no process pinning, performance will likely be suboptimal.\n");
    }
}

void Utils::check_readable(const char *filename) {
    if (access(filename, R_OK) == -1)
        perror("Cannot read input file");
}

void Utils::print_license() {
    printf(Utils::license);
}

void Utils::print_citation() {
    printf(Utils::citation);
}

void Utils::print_copyright() {
    printf("\n(c) %s %s\n\n", Utils::years, Utils::author);

}
