#ifndef SLIDER_UTILS_H
#define SLIDER_UTILS_H

#include <cstring>
#include <sstream>
#include <fstream>
#include <stdexcept>
#include <cmath>
#include <sys/stat.h>
#include <unistd.h>
#include <omp.h>

/*
 * MPI_CHECK will display a custom error message as well as an error string
 * from the MPI_STATUS and then exit the program
 * SOURCE: https://github.com/LLNL/ior/blob/master/src/iordef.h
 */

#define MPI_CHECK(MPI_STATUS, MSG) do {                                  \
    char resultString[MPI_MAX_ERROR_STRING];                             \
    int resultLength;                                                    \
                                                                         \
    if (MPI_STATUS != MPI_SUCCESS) {                                     \
        MPI_Error_string(MPI_STATUS, resultString, &resultLength);       \
        fprintf(stdout, "Sparsify ERROR: %s, MPI %s, (%s:%d)\n",              \
                MSG, resultString, __FILE__, __LINE__);                  \
        fflush(stdout);                                                  \
        MPI_Abort(MPI_COMM_WORLD, -1);                                   \
    }                                                                    \
} while(0)


namespace Utils {
    const char author[] = "Krzysztof Młynarczyk";
    const char years[] = "2017-2019";
    const char license[] = "This program is free to use for non-commercial entities only\n";
    const char citation[] = "If the results obtained by using this application were used for publication, please cite:\n"
                        "Merski M. et al. Self-Analysis of Repeat Proteins Reveals Evolutionarily\n"
                        "Conserved Patterns (in review)";

    std::string generateFileName(const char *prefix, const char *suffix, const char *ext);

    off64_t getFilesize(const char *filename);

    void check_readable(const char *filename);

    void printInfo(int argc, char *const *argv, int nWorld);

    void print_license();

    void print_citation();

    void print_copyright();

    void check_threads(const int &expectedNThreads);

    void get_proc_bind_info();

    void jobIndexToDotPlotIndices(const size_t &nDotPlots, const int_fast64_t &jobIndex, uint &row, uint &col);
}
#endif //SLIDER_UTILS_H
